#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import unittest
from config import HBTConfig
from actions import CommitCheckAction, Action
from sourcefile import SourceFile, JavaFile, XmlFile

JAVA_CONTENT = """\
package com.wlqq.data.net;

import com.raizlabs.android.dbflow.annotation.NotNull;
import com.wlqq.data.request.LoginParams;
import com.wlqq.login.model.Session;

import java.util.Map;

import io.reactivex.Observable;

/**
 * Interfaces of gas station.
 */
public interface IGasApi {
    /**
     * Get an {@link Observable} which will emit a {@link Session}.
     */
    Observable<Session> login(@NotNull Map<String, String> headers, @NotNull LoginParams params);
    void logout();
}
"""

XML_CONTENT = """\
<?xml version="1.0" encoding="UTF-8"?>
<!-- Copyright (C) 2017 贵阳货车帮科技有限公司 -->
<!DOCTYPE mapper PUBLIC "-//mybatis.org//DTD Mapper 3.0//EN"
        "http://mybatis.org/dtd/mybatis-3-mapper.dtd">

<mapper namespace="com.wlqq.dms.api.build.dao.BuildLogPatternDao">
    <sql id="column">
        uuid,
        pattern,
        error_code AS errorCode,
        error_message AS errorMessage,
        create_user AS createUser,
		create_time AS createTime,
		update_user AS updateUser,
		update_time AS updateTime
    </sql>

    <select id="list" resultType="com.wlqq.dms.api.build.domain.BuildLogPattern">
        SELECT * from dms_build_log_pattern
    </select>
</mapper>
"""


def split_file_path(p):
    return [os.path.dirname(p), os.path.basename(p)]


class Hbt2TestCase(unittest.TestCase):
    java_file_path = 'TestJava.java'
    xml_file_path = 'testXml.xml'

    def setUp(self):
        with open(self.java_file_path, 'w') as f:
            f.write(JAVA_CONTENT)
        with open(self.xml_file_path, 'w') as f:
            f.write(XML_CONTENT)

    def tearDown(self):
        for p in [self.java_file_path, self.xml_file_path]:
            if os.path.exists(p):
                os.remove(p)

    def test_config(self):
        default_config = HBTConfig('hbtrc')

        self.assertEqual(default_config.jira_url, "http://jira.56qq.cn")
        self.assertEqual(default_config.reviewboard_url, "http://reviewboard.56qq.com")
        self.assertEqual(default_config.clean_check, True)
        self.assertEqual(default_config.commit_message_check, True)
        self.assertEqual(default_config.checkstyle, True)
        self.assertEqual(default_config.jira_user, "user")
        self.assertEqual(default_config.jira_password, "password")
        self.assertEqual(default_config.hbt_dir, "/a/b/c")

    def test_commited_files(self):
        commited_log = """
M       .gitignore
A       doc/dms_20171017.sql
M       src/main/java/com/wlqq/dms/api/login/controller/LoginController.java
M       src/main/java/com/wlqq/dms/api/login/vo/LoginResponseVO.java
M       src/main/java/com/wlqq/dms/api/user/controller/UserController.java
M       src/main/java/com/wlqq/dms/api/user/dao/UserDao.java
M       src/main/java/com/wlqq/dms/api/user/domain/User.java
M       src/main/java/com/wlqq/dms/api/user/service/UserService.java
M       src/main/java/com/wlqq/dms/api/user/service/impl/UserServiceImpl.java
A       src/main/java/com/wlqq/dms/api/user/vo/UserApiKeyUpdateVO.java
M       src/main/resources/mapper/user/UserDaoMapper.xml
        """
        files = Action(None, None).get_commit_file_list(commited_log.split('\n'))
        self.assertEqual(len(files), 9)
        self.assertFalse(filter(lambda x: not x.endswith('.java') and not x.endswith('.xml'), files))

        rename_log = """R100    code_analysis/hbt/scripts/test.java      code_analysis/hbt/scripts/test1.java"""
        files = Action(None, None).get_commit_file_list(rename_log.split('\n'))
        print(files)

    def test_sourcefile_op(self):
        sf = SourceFile.create(*split_file_path(self.java_file_path))
        self.assertTrue(isinstance(sf, JavaFile))
        sf.add_copyright()

        with open(self.java_file_path) as f:
            content = f.read()
            self.assertTrue(content.startswith(sf.get_copyright_text()))

        sf = SourceFile.create(*split_file_path(self.xml_file_path))
        self.assertTrue(isinstance(sf, XmlFile))
        sf.add_copyright()

        with open(self.xml_file_path) as f:
            content = f.read()
            self.assertTrue(content.startswith(sf.get_copyright_text()))

    def test_commit_message(self):
        config = HBTConfig('hbtrc')
        config.commit_message_check = True

        valid_messages = [
            """\
fix(ANDROID_INFRA-57): summary

body

测试影响\
""",
            """\
fix(ANDROID_INFRA-57): summary

body

影响：启动页面

http://reviewboard.56qq.com/r/3889/
http://jira.56qq.cn/browse/ANDROID_INFRA-96\
""",
            """\
fix(ANDROID_INFRA-57): summary

body

测试影响
""",
            """\
feat(ANDROID_INFRA-57): summary

body

测试影响
            """,
            """\
docs(ANDROID_INFRA-57): summary

body

测试影响
""",
            """\
test(ANDROID_INFRA-57): summary

body

body2

测试影响
""",
        ]
        action = CommitCheckAction(config, None)
        for msg in valid_messages:
            action.commit_message = msg
            self.assertFalse(action.do())

        invalid_messages = [
            """\
unknownfeat(ANDROID_INFRA-57): summary

body

测试影响
                        """,
            """\
unknownfeat(ANDROID_INFRA-57): summary

测试影响
                        """,
            """\
unknownfeat(ANDROID_INFRA-57): summary
测试影响
                        """,
            """\
unknownfeat(ANDROID_INFRA-57): summary

测试影响
                                    """,
            """\
unknownfeat(ANDROID_INFRA-57): summary

这里是正文
            """,
            """\
这里是正文

测试影响
            """,
            """\
feat ANDROID_INFRA-57 summary

测试影响
            """
    ]
        for msg in invalid_messages:
            action.commit_message = msg
            self.assertTrue(action.do())

    def test_review_url(self):
        config = HBTConfig('hbtrc')
        request_id = '3889'
        review_url = 'http://reviewboard.56qq.com/r/%s/' % request_id
        commit_message = """\
fix(ANDROID_INFRA-57): summary

body

影响：启动页面

http://reviewboard.56qq.com/r/3889/
http://jira.56qq.cn/browse/ANDROID_INFRA-96
        """
        action = Action(config, None, commit_message)
        self.assertEqual((review_url, request_id), action.get_review_url())

        commit_message = """\
        fix(ANDROID_INFRA-57): summary

        body

        影响：启动页面
                """
        action = Action(config, None, commit_message)
        self.assertEqual((None, None), action.get_review_url())

    def test_commit_file_list(self):
        commit_javas = """\
M   src/main/java/com/wlqq/checkout/CheckOutActivity.java
D   src/main/java/com/wlqq/shift/activity/FixShiftSuccessActivity.java
A   src/main/java/com/wlqq/shift/activity/UnConfirmedShiftActivity.java
M   src/main/res/values/values.xml
D   src/main/res/values/colors.xml
A   src/main/res/values/strings.xml
R100    debug.java  test.java
R109    src.xml  dest.xml
            """
        self.assertEqual(6, len(Action(None, None, None).get_commit_file_list(commit_javas.split('\n'))))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from jira import JIRA, JIRAError
from rbtools.api.client import RBClient
import subprocess
import re
import os
import platform
from sourcefile import SourceFile
from hbt import check_commit_message_conventions


SUCCESS = ''


class Action(object):
    subject_re = r'^\s*(feat|fix|docs|style|refactor|chore)?\s*\((.*)\)\s*[:|：]\s*(.+)\n'
    new_line_re = r'\n'
    body_re = r'((?:.+\n)+)'
    affect_re = r'^([测试影响|影响].*)'
    commit_re = subject_re + new_line_re + body_re + new_line_re + affect_re

    rename_re = r'^\s*R\d+\s+.+[\.java|\.xml]\s+(.+(\.java|\.xml))\s*$'
    update_re = r'^\s*[M|A]\s+(.+(\.java|\.xml))\s*$'

    GROUP_INDEX_FILE_NAME = 1

    STATUS_SUBMITTED = 'submitted'

    def __init__(self, config, ctx, commit_message=''):
        self.config = config
        self.context = ctx
        self.debug = ctx.params['debug'] if ctx else False
        self.commit_message = commit_message or subprocess.check_output('git log --format=%B -n 1', shell=True).strip()

    def get_issue_id(self):
        return re.compile(self.subject_re).match(self.commit_message).group(2)

    def get_review_request_by_id(self, request_id):
        client = RBClient(self.config.reviewboard_url)
        return client.get_root().get_review_request(review_request_id=request_id)

    def get_review_url(self, lines=None):
        if not lines:
            lines = self.commit_message.split('\n')
        review_request_pattern = re.compile(r'^\s*%s/r/(\d+)/\s*$' % self.config.reviewboard_url)
        for line in lines:
            result = review_request_pattern.match(line)
            if result:
                return line, result.group(1)
        else:
            print(u'[WARN]: 没有解析到reviewboard url')
            return None, None

    def get_committed(self):
        commit_log = subprocess.check_output('git log --name-status --pretty=format:"" -n 1', shell=True)
        return [SourceFile.create(os.path.dirname(f), os.path.basename(f)) for f in
                self.get_commit_file_list(commit_log.split('\n'))]

    def get_commit_file_list(self, lines):
        files = list()
        rename_pattern = re.compile(self.rename_re)
        update_pattern = re.compile(self.update_re)
        for line in lines:
            result = rename_pattern.match(line)
            if result:
                files.append(result.group(self.GROUP_INDEX_FILE_NAME))
                continue

            result = update_pattern.match(line)
            if result:
                files.append(result.group(self.GROUP_INDEX_FILE_NAME))

        return files

    def amend_message(self):
        return SUCCESS if subprocess.call('git commit -a --amend --no-edit', shell=True) == 0 else \
               u'更新commit message失败'

    def printDebugLog(self, message):
        if self.debug:
            print message


class RbtPostArgsCheckAction(Action):
    branch_param_re = r'--branch.*'
    def do(self):
        if not self.config.branch_check:
            return SUCCESS
        self.printDebugLog('[hbt2] 命令参数合法性检查中...')
        branch_param_pattern = re.compile(self.branch_param_re)
        for arg in self.context.params['rbt_args']:
            if branch_param_pattern.match(arg):
                return SUCCESS
        return u'WARNING: 必须指定远程branch名称，请使用--branch来指定'

class WorkspaceCleanAction(Action):
    def do(self):
        self.printDebugLog('[hbt2] Workspace状态检查中...')
        if self.config.clean_check and \
                '' != subprocess.check_output('git status --porcelain --untracked-files=no', shell=True):
            return u'WARNING: 有未commit的文件。请commit后再发送review request'
        return SUCCESS


class CommitCheckAction(Action):
    def do(self):
        self.printDebugLog('[hbt2] Commit Message合法性检查中...')
        if check_commit_message_conventions(self.commit_message.split('\n')) == None:
            return u'WARNING: commit message格式不合格，请按提示修正。'
        return SUCCESS



class LandCheckAction(Action):
    def do(self):
        review_url, request_id = self.get_review_url()
        if not request_id:
            return u'未能查询到review request id'

        print(u'开始检查review request是否能够提交： ' + request_id)
        review_request = self.get_review_request_by_id(request_id)

        if not review_request.approved:
            print(review_request.approval_failure)
            return u'reviewboard检查失败'

        if not self.exist_remote_branch():
            return u'不存在的远程分支。请检查分支名。'

        return SUCCESS

    def exist_remote_branch(self):
        remote_name = 'remotes/origin/{}'.format(self.context.params['branch'])

        for line in subprocess.check_output('git branch -a', shell=True).split('\n'):
            if line.strip() == remote_name:
                return True
        else:
            return False


class PushAction(Action):
    def do(self):
        return SUCCESS if subprocess.call('git push origin HEAD:' + self.context.params['branch'], shell=True) == 0 \
                       else u'push失败'


class JiraUpdateAction(Action):
    def do(self):
        authed_jira = JIRA(server=self.config.jira_url, basic_auth=(self.config.jira_user, self.config.jira_password))
        issue = authed_jira.issue(self.get_issue_id())
        authed_jira.add_comment(issue, self.commit_message)

        # '4' 代表状态是 开发中
        try:
            authed_jira.transition_issue(issue, transition='4')
        except JIRAError:
            # 当jira状态已经为“开发中”时，这里transition会抛异常。ignore it
            pass
        return SUCCESS


class CloseReviewboardAction(Action):
    def do(self):
        commit_id = subprocess.check_output('git log --format="%H" -n 1', shell=True)
        description = 'hbt closed: {}'.format(commit_id)

        _, request_id = self.get_review_url()
        if not request_id:
            return u'未能查询到review request id'

        review_request = self.get_review_request_by_id(request_id)
        if review_request.status == self.STATUS_SUBMITTED:
            return u'Review request {} is already submitted.'.format(request_id)

        review_request.update(status=self.STATUS_SUBMITTED, description=description)
        return SUCCESS


class CopyrightUpdateAction(Action):
    def do(self):
        self.printDebugLog('[hbt2] Copyright自动处理中...')
        for f in self.get_committed():
            f.add_copyright()
        self.amend_message()
        return SUCCESS


class StyleCheckAction(Action):
    CYGWIN_SYSTEM_PREFIX = 'CYGWIN'
    CHECKSTYLE_JAR_FILE_NAME = 'checkstyle-all.jar'
    CHECKSTYLE_CONFIG_FILE_NAME = 'checkstyle-config.xml'

    def do(self):
        self.printDebugLog('[hbt2] 代码风格检查...')
        return SUCCESS if self.config.checkstyle or self.style_errors() == 0 else\
            '\nWARNING: You must fix the errors and warnings first, then post review again'

    def style_errors(self):
        hbt_dir = self.config.hbt_dir
        if platform.system().startswith(self.CYGWIN_SYSTEM_PREFIX):
            # change to cygwin path
            hbt_dir = subprocess.check_output('cygpath -m "' + self.config.hbt_dir + '"', shell=True).strip()

        jar_file = os.path.join(hbt_dir, self.CHECKSTYLE_JAR_FILE_NAME)
        config_file = os.path.join(hbt_dir, self.CHECKSTYLE_CONFIG_FILE_NAME)

        # check code command
        command = 'java -jar ' + jar_file + ' -c ' + config_file

        for f in self.get_committed():
            if f.check_style(command) != 0:
                return 1
        return 0


class RBTPostAction(Action):
    CHECKSTYLE_TAG = '[checked_5_0_1]'

    def do(self):
        args = self.context.params['rbt_args']
        rbt_command = 'rbt post ' + ' '.join(arg for arg in args)
        summary = self.commit_message.split('\n')[0].replace('"', '\\\"')
        if summary:
            rbt_command += ' --summary "%s %s"' % (self.CHECKSTYLE_TAG, summary)
        rbt_command = rbt_command.replace(' -s ', '')
        need_auto_open = rbt_command.find(' -o')
        rbt_command = rbt_command.replace(' -o', '')
        if self.debug:
            rbt_command += ' -d'

        self.printDebugLog('[hbt2] ' + rbt_command)
        post_message = subprocess.check_output(rbt_command, shell=True)

        review_url, request_id = self.get_review_url(post_message.split('\n'))
        if not review_url:
            return u'WARNING: 没有找到review url信息'

        # 再次更新 review request 的描述以加入新增的 jira url 和 review request link 信息等
        if self.update_commit_message(review_url) == 0:
            rbt_command += ' -r ' + request_id
            if need_auto_open:
                rbt_command += ' -o'

            self.printDebugLog('[hbt2] ' + rbt_command)
            return SUCCESS if subprocess.call(rbt_command, shell=True) == 0 else\
                u'WARNING: rbt命令执行出错'

        return u'WARNING: 更新commit message失败'

    def update_commit_message(self, review_url):
        lines = []
        has_updated = False
        review_request_re = r'^\s*%s/r/(\d+)/\s*$'
        jira_link_re = r'^\s*%s/browse/.*\s*$'
        jira_link = '%s/browse/%s' % (self.config.jira_url, self.get_issue_id())

        for line in self.commit_message.split('\n'):
            if re.compile(review_request_re % self.config.reviewboard_url).match(line):
                lines.append(review_url)
                has_updated = True
            elif re.compile(jira_link_re % self.config.jira_url).match(line):
                lines.append(jira_link)
            else:
                lines.append(line.decode('utf-8'))

        if not has_updated:
            lines.append('')
            lines.append(review_url)
            lines.append(jira_link)
        new_message = '\n'.join(lines).replace('"', '\\\"')
        # update review board url to commit message
        return subprocess.call('git commit --amend -m "%s"' % new_message, shell=True)


class RBTAction(Action):
    def do(self):
        self.printDebugLog(u'[hbt2] 执行rbt...')
        args = self.context.params['rbt_args']
        rbt_command = 'rbt %s %s' % (self.context.info_name.encode('ascii', 'ignore'), ' '.join(arg for arg in args))
        if self.debug:
            rbt_command += ' -d'
        self.printDebugLog('[hbt2] ' + rbt_command)
        return SUCCESS if subprocess.call(rbt_command, shell=True) == 0 else\
            'WARNING: rbt命令执行出错'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import traceback
from actions import *


class InvalidCommitError(BaseException):
    """Raised if an invalid configuration is encountered."""

    def __init__(self, message):
        super(InvalidCommitError, self).__init__(message)

profile_rbt_land = 'land'
profile_rbt_post = 'post'
profile_rbt = 'rbt'
pipeline_rbt_land = [WorkspaceCleanAction, CommitCheckAction, LandCheckAction, PushAction, JiraUpdateAction, CloseReviewboardAction]
pipeline_rbt_post = [WorkspaceCleanAction, CommitCheckAction, CopyrightUpdateAction, StyleCheckAction, RbtPostArgsCheckAction, RBTPostAction]
pipeline_rbt = [RBTAction]

profiles = {
    profile_rbt_land: pipeline_rbt_land,
    profile_rbt_post: pipeline_rbt_post,
    profile_rbt: pipeline_rbt
}


class Pipeline(object):
    def __init__(self, config, ctx, actions_class):
        self.queue = []
        commit_message = subprocess.check_output('git log --format=%B -n 1', shell=True).strip()
        for action in actions_class:
            self.queue.append(action(config, ctx, commit_message))

    def run(self):
        for action in self.queue:
            try:
                err = action.do()
                if err:
                    print(err)
                    sys.exit(1)
            except Exception as e:
                traceback.print_exc()
                sys.exit(1)

    @staticmethod
    def create_pipeline(config, ctx, profile):
        return Pipeline(config, ctx, profiles.get(profile))


if __name__ == '__main__':
    subject_re = r'^\s*(feat|fix|docs|style|refactor|chore)?\s*\((.*)\)\s*[:|：]\s*(.+)\n'
    new_line_re = r'\n'
    body_re = r'((?:.+\n)+)'
    affect_re = r'^([测试影响|影响].*)\n'

    msg = """feat(ANDROID_INFRA-57): 修改copyright后自动commit文件。消除手动commit的困扰。

扫描被修改的文件，发现copyright不对的，进行修改。然后自动提交commit相关改动。
扫描被修改的文件，发现copyright不对的，进行修改。然后自动提交commit相关改动。
扫描被修改的文件，发现copyright不对的，进行修改。然后自动提交commit相关改动。

测试影响： copyright 相关测试

rev
"""
    # print re.compile(subject_re + new_line_re + body_re + new_line_re + affect_re, re.MULTILINE).match(msg).group(2)
    print (re.compile(subject_re, re.MULTILINE).match(msg).group(2))
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import click
import os

from sourcefile import SourceFile
from config import HBTConfig
import pipeline

RBT_COMMAND_NAME = 'rbt'
CONTEXT_SETTINGS = dict(token_normalize_func=lambda x: RBT_COMMAND_NAME)

config = HBTConfig()


class AliasedGroup(click.Group):
    def get_command(self, ctx, cmd_name):
        return click.Group.get_command(self, ctx, cmd_name) or click.Group.get_command(self, ctx, RBT_COMMAND_NAME)


@click.command(cls=AliasedGroup)
@click.version_option()
def cli():
    pass


@click.command(short_help='auto update copyright')
@click.argument('source_dir')
def copyright(source_dir):
    to_parse = list()
    for root, dirs, files in os.walk(source_dir):
        for file in files:
            source_file = SourceFile.create(root, file)
            if source_file:
                to_parse.append(source_file)
    [(click.echo(f.name), f.add_copyright()) for f in to_parse]


@click.command(context_settings=dict(ignore_unknown_options=True,),
    help='use \'rbt post --help\' to see help text')
@click.argument('rbt_args', nargs=-1, type=click.UNPROCESSED)
@click.option('-d', '--debug', is_flag=True, help='Displays debug output.')
@click.pass_context
def post(ctx, rbt_args, debug):
    pipeline.Pipeline.create_pipeline(config, ctx, pipeline.profile_rbt_post).run()


@click.command(context_settings=dict(ignore_unknown_options=True,),
    help='use \'rbt land --help\' to see help text', options_metavar='')
@click.argument('branch', required=True)
@click.argument('rbt_args', nargs=-1, type=click.UNPROCESSED)
@click.option('-d', '--debug', is_flag=True, help='Displays debug output.')
@click.pass_context
def land(ctx, branch, rbt_args, debug):
    pipeline.Pipeline.create_pipeline(config, ctx, pipeline.profile_rbt_land).run()


@click.command(context_settings=dict(ignore_unknown_options=True, allow_extra_args=True),
    short_help='use \'rbt --help\' to see more',
    help='use \'rbt <rbt_command> --help\' to see help text')
@click.argument('rbt_args', nargs=-1, type=click.UNPROCESSED)
@click.option('-d', '--debug', is_flag=True, help='Displays debug output.')
@click.pass_context
def rbt(ctx, rbt_args, debug):
    pipeline.Pipeline.create_pipeline(config, ctx, pipeline.profile_rbt).run()


def main():
    cli.add_command(copyright)
    cli.add_command(post)
    cli.add_command(land)
    cli.add_command(rbt)
    cli()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import subprocess
import re
import time

from jira import JIRA, JIRAError
from rbtools.api.client import RBClient


class SourceFile(object):
    COPYRIGHT_TEMPLATE = '%s'

    TYPE_JAVA = 'java'
    TYPE_XML = 'xml'
    TYPE_OTHERS = 'others'

    LEVEL_ERROR = 'ERROR'
    LEVEL_WARN = 'WARN'

    BODY_PATTERN = None

    def __init__(self, path, name):
        self.path = path
        self.name = name
        self.abs_path = os.path.join(path, name)
        self.created = self.get_create_year()
        self.current = self.get_current_year()

    @staticmethod
    def create(path, name):
        if name.endswith(SourceFile.TYPE_JAVA):
            return JavaFile(path, name)
        elif name.endswith(SourceFile.TYPE_XML):
            return XmlFile(path, name)
        else:
            return None

    def check_style(self, command):
        return 0

    def get_level_count(self, lines, level):
        return len(re.compile(r"\[%s\]" % level).findall(lines))

    def add_copyright(self):
        with open(self.abs_path) as f:
            lines = self.remove_leading_comment(f.readlines())

        with open(self.abs_path, 'w') as f:
            f.write(self.get_copyright_text())
            f.writelines(lines)

    def get_create_year(self):
        command = 'git log --follow --format=%ai --reverse -- {} | head -1'.format(self.abs_path)

        # output example: 2015-11-10 15:44:29 +0800
        date_format = subprocess.check_output(command, shell=True)

        if len(date_format) == 0:
            return self.get_current_year()

        return date_format.split('-')[0]

    def get_current_year(self):
        return time.strftime('%Y', time.localtime(time.time()))

    def remove_leading_comment(self, lines):
        idx = 0
        for idx, line in enumerate(lines):
            if self.BODY_PATTERN.match(line):
                break
        return lines[idx:]

    def get_copyright_text(self):
        return self.COPYRIGHT_TEMPLATE % \
               (self.current if self.created == self.current else
                '%s - %s' % (self.created, self.current))


class JavaFile(SourceFile):
    COPYRIGHT_TEMPLATE = """\
/*
 * Copyright (C) %s 贵阳货车帮科技有限公司
 */\n
"""
    BODY_PATTERN = re.compile(r'^package.+$')

    def check_style(self, command):
        result = subprocess.check_output(command + ' ' + self.abs_path, shell=True)
        return self.get_level_count(result, self.LEVEL_ERROR) + self.get_level_count(result, self.LEVEL_WARN)


class XmlFile(SourceFile):
    COPYRIGHT_TEMPLATE = """\
<?xml version="1.0" encoding="utf-8"?>
<!-- Copyright (C) %s 贵阳货车帮科技有限公司 -->\n"""
    BODY_PATTERN = re.compile(r'^<(!DOCTYPE|[a-zA-Z]).+$')


class WorkSpace(object):
    SUBJECT_PATTERN = r'^\s*(feat|fix|docs|style|refactor|chore)?\s*\((.*)\)\s*[:|：]\s*(.+)$'
    AFFECT_PATTERN = r'^[测试影响|影响]'
    REVIEW_REQUEST_PATTERN = r'^\s*%s/r/(\d+)/\s*$'
    RENAME_PATTERN = r'^\s*R\d+\s+.+[\.java|\.xml]\s+(.+[\.java|\.xml])\s*$'
    ADD_OR_MODIFY_PATTERN = r'^\s*[M|A]\s+(.+[\.java|\.xml])\s*$'

    GROUP_INDEX_FILE_NAME = 1

    STATUS_SUBMITTED = 'submitted'

    def __init__(self, config):
        self.config = config
        self.issue_id = None
        self.commit_message = subprocess.check_output('git log --format=%B -n 1', shell=True).strip().split('\n')
        self.review_url, self.request_id = self.get_review_url()

    def is_clean(self):
        return self.config.clean_check and \
               '' != subprocess.check_output('git status --porcelain --untracked-files=no', shell=True)

    def get_review_url(self):
        pattern = re.compile(self.REVIEW_REQUEST_PATTERN % self.config.reviewboard_url)
        for line in self.commit_message:
            result = pattern.match(line)
            if result:
                return line, result.group(1)
        else:
            print('[WARN]: 没有解析到reviewboard url')
            return None, None

    def check_commit_message(self):
        if not self.config.commit_message_check:
            return ''

        if not self.commit_message:
            return u'commit message为空'

        result = self.check_subject(self.commit_message[0])
        if result:
            return result

        if not self.config.strict:
            # 非严格模式，不检查body和测试影响
            return ''

        result = self.check_affect(self.commit_message)
        if result:
            return result

        result, reason = self.check_body(self.commit_message[1:-1])
        if not result:
            return False, reason

        return True, ''

    def check_subject(self, subject):
        result = re.compile(self.SUBJECT_PATTERN).match(subject)
        reason = u'第一行为subject,格式为 <type>(<issue_id>): <subject>'
        if result:
            self.issue_id = result.group(2)
            reason = ''
        return reason

    def check_body(self, lines):
        if len(lines) < 3:
            return False, u"commit message缺少<subject>, <body>, <affect>元素之一"
        if len(lines) > 1 and lines[1].strip():
            return False, u"标题的下面应该为空行"

        if lines[-2].strip():
            return False, u'"测试影响"与body之间需要空行'
        if lines[1].strip():
            return False, u'subject与body之间需要空行'
        return True, ''

    def check_affect(self, lines):
        pattern = re.compile(self.AFFECT_PATTERN)
        for idx, line in enumerate(lines):
            if pattern.match(line):
                break
        else:
            return u'没有"测试影响"描述'

        if lines[idx - 1].strip():
            return u'"测试影响"与body之间需要空行'
        if idx < 4:
            return u'缺少<body>'

        return ''

    def get_commit_message(self):
        self.commit_message = subprocess.check_output('git log --format=%B -n 1', shell=True).strip().split('\n')
        return self.commit_message

    def check_land_request(self):
        if not self.request_id:
            return u'未能查询到review request id'

        print(u'开始检查review request是否能够提交： ' + self.request_id)
        review_request = self.get_review_request_by_id()

        if not review_request.approved:
            return review_request.approval_failure
        return ''

    def get_review_request_by_id(self):
        client = RBClient(self.config.reviewboard_url)
        return client.get_root().get_review_request(review_request_id=self.request_id)

    def exist_remote_branch(self, name):
        lines = subprocess.check_output('git branch -a', shell=True).split('\n')
        remote_name = 'remotes/origin/{}'.format(name)

        for line in lines:
            if line.strip() == remote_name:
                return True
        else:
            return False

    def push_change(self, branch_name):
        return subprocess.call('git push origin HEAD:' + branch_name, shell=True)

    def update_jira_comment(self):
        jira_user = subprocess.check_output('git config jira.user', shell=True).strip() or self.config.user
        jira_password = subprocess.check_output('git config jira.pwd', shell=True).strip() or self.config.password

        authed_jira = JIRA(server=(self.config.jira_url), basic_auth=(jira_user, jira_password))
        if not self.issue_id:
            return False
        issue = authed_jira.issue(self.issue_id)
        authed_jira.add_comment(issue, '\n'.join(self.commit_message))

        # '4' 代表状态是 开发中
        try:
            authed_jira.transition_issue(issue, transition='4')
        except JIRAError:
            # 当jira状态已经为“开发中”时，这里transition会抛异常。ignore it
            return False
        return True

    def close_review_request(self):
        commit_id = subprocess.check_output('git log --format="%H" -n 1', shell=True)
        description = 'hbt closed: {}'.format(commit_id)

        review_request = self.get_review_request_by_id()
        if review_request.status == self.STATUS_SUBMITTED:
            print('Review request {} is already submitted.'.format(self.request_id))
            return

        review_request.update(status=self.STATUS_SUBMITTED, description=description)

    def get_commit_file_list(self, lines):
        files = list()
        rename_pattern = re.compile(self.RENAME_PATTERN)
        update_pattern = re.compile(self.ADD_OR_MODIFY_PATTERN)
        for line in lines:
            result = rename_pattern.match(line)
            if result:
                files.append(result.group(self.GROUP_INDEX_FILE_NAME))
                continue

            result = update_pattern.match(line)
            if result:
                files.append(result.group(self.GROUP_INDEX_FILE_NAME))

        return files

    def committed_files(self):
        commit_log = subprocess.check_output('git log --name-status --pretty=format:"" -n 1', shell=True)
        return [SourceFile.create(os.path.dirname(f), os.path.basename(f))
                for f in self.get_commit_file_list(commit_log.split('\n'))]

    def ammend_message(self):
        subprocess.call('git commit -a --amend --no-edit', shell=True)


if __name__ == '__main__':
    java = SourceFile.create('.', 'TestNoWarn.java')
    print(java.path, java.name)
    java.add_copyright()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from os.path import expanduser, abspath, dirname, join
import os
import io
import json

import six

DEFAULT_CONFIG_LOCATION = expanduser("~/.hbtrc")

DEFAULT_CONFIG = {
    'jira_url': 'https://jira.56qq.cn',
    'reviewboard_url': 'http://reviewboard.56qq.com',
    'clean_check': True,
    'commit_message_check': True,
    'checkstyle': True,
    'hbt_dir': dirname(abspath(__file__)),
    'jira_user': '',
    'jira_password': '',
    'branch_check': True
}


class InvalidConfigError(ValueError):
    """Raised if an invalid configuration is encountered."""

    def __init__(self, message):
        super(InvalidConfigError, self).__init__(message)


class HBTConfig(object):
    def __init__(self, filename=None):
        if filename is None and os.path.isfile(DEFAULT_CONFIG_LOCATION):
            filename = DEFAULT_CONFIG_LOCATION

        self.override(DEFAULT_CONFIG)
        if filename is not None:
            try:
                with io.open(filename, encoding='utf-8') as f:
                    file_config = json.loads(f.read())
            except ValueError as e:
                raise InvalidConfigError("Failed to read configuration file '{}'. Error: {}".format(filename, e))
            self.override(file_config)
            if not (self.jira_user and self.jira_password and self.hbt_dir):
                raise InvalidConfigError(u"jira_user, jira_password, hbt_dir不能为空")

    # noinspection PyCompatibility
    def make_unicode(self, config):
        if six.PY2:
            # Sometimes (depending on the source of the config value) an argument will be str instead of unicode
            # to unify that and ease further usage of the config, we convert everything to unicode
            for k, v in config.items():
                if type(v) is str:
                    config[k] = unicode(v, "utf-8")
        return config

    def override(self, config):
        # abs_path_config = self.make_unicode(self.make_paths_absolute(config, ["path", "response_log"]))
        self.__dict__.update(config)

    def make_paths_absolute(self, config, keys):
        abs_path_config = dict(config)
        for key in keys:
            if key in abs_path_config and abs_path_config[key] is not None and not os.path.isabs(abs_path_config[key]):
                abs_path_config[key] = join(os.getcwd(), abs_path_config[key])
        return abs_path_config


if __name__ == '__main__':
    c = HBTConfig()
    print c
